"""
This type stub file was generated by pyright.
"""

r"""A module for console attributes, special characters and functions.

The target architectures {linux, macos, windows} support inline encoding for
all attributes except color. Windows requires win32 calls to manipulate the
console color state.

Usage:

  # Get the console attribute state.
  out = log.out
  con = console_attr.GetConsoleAttr(out=out)

  # Get the ISO 8879:1986//ENTITIES Box and Line Drawing characters.
  box = con.GetBoxLineCharacters()
  # Print an X inside a box.
  out.write(box.dr)
  out.write(box.h)
  out.write(box.dl)
  out.write('\n')
  out.write(box.v)
  out.write('X')
  out.write(box.v)
  out.write('\n')
  out.write(box.ur)
  out.write(box.h)
  out.write(box.ul)
  out.write('\n')

  # Print the bullet characters.
  for c in con.GetBullets():
    out.write(c)
  out.write('\n')

  # Print FAIL in red.
  out.write('Epic ')
  con.Colorize('FAIL', 'red')
  out.write(', my first.')

  # Print italic and bold text.
  bold = con.GetFontCode(bold=True)
  italic = con.GetFontCode(italic=True)
  normal = con.GetFontCode()
  out.write('This is {bold}bold{normal}, this is {italic}italic{normal},'
            ' and this is normal.\n'.format(bold=bold, italic=italic,
                                            normal=normal))

  # Read one character from stdin with echo disabled.
  c = con.GetRawKey()
  if c is None:
    print 'EOF\n'

  # Return the display width of a string that may contain FontCode() chars.
  display_width = con.DisplayWidth(string)

  # Reset the memoized state.
  con = console_attr.ResetConsoleAttr()

  # Print the console width and height in characters.
  width, height = con.GetTermSize()
  print 'width={width}, height={height}'.format(width=width, height=height)

  # Colorize table data cells.
  fail = console_attr.Colorizer('FAIL', 'red')
  pass = console_attr.Colorizer('PASS', 'green')
  cells = ['label', fail, 'more text', pass, 'end']
  for cell in cells;
    if isinstance(cell, console_attr.Colorizer):
      cell.Render()
    else:
      out.write(cell)
"""
class BoxLineCharacters:
  """Box/line drawing characters.

  The element names are from ISO 8879:1986//ENTITIES Box and Line Drawing//EN:
    http://www.w3.org/2003/entities/iso8879doc/isobox.html
  """
  ...


class BoxLineCharactersUnicode(BoxLineCharacters):
  """unicode Box/line drawing characters (cp437 compatible unicode)."""
  dl = ...
  dr = ...
  h = ...
  hd = ...
  hu = ...
  ul = ...
  ur = ...
  v = ...
  vh = ...
  vl = ...
  vr = ...
  d_dl = ...
  d_dr = ...
  d_h = ...
  d_hd = ...
  d_hu = ...
  d_ul = ...
  d_ur = ...
  d_v = ...
  d_vh = ...
  d_vl = ...
  d_vr = ...


class BoxLineCharactersAscii(BoxLineCharacters):
  """ASCII Box/line drawing characters."""
  dl = ...
  dr = ...
  h = ...
  hd = ...
  hu = ...
  ul = ...
  ur = ...
  v = ...
  vh = ...
  vl = ...
  vr = ...
  d_dl = ...
  d_dr = ...
  d_h = ...
  d_hd = ...
  d_hu = ...
  d_ul = ...
  d_ur = ...
  d_v = ...
  d_vh = ...
  d_vl = ...
  d_vr = ...


class BoxLineCharactersScreenReader(BoxLineCharactersAscii):
  dl = ...
  dr = ...
  hd = ...
  hu = ...
  ul = ...
  ur = ...
  vh = ...
  vl = ...
  vr = ...


class ProgressTrackerSymbols:
  """Characters used by progress trackers."""
  ...


class ProgressTrackerSymbolsUnicode(ProgressTrackerSymbols):
  """Characters used by progress trackers."""
  @property
  def spin_marks(self): # -> list[str]:
    ...
  
  success = ...
  failed = ...
  interrupted = ...
  not_started = ...
  prefix_length = ...


class ProgressTrackerSymbolsAscii(ProgressTrackerSymbols):
  """Characters used by progress trackers."""
  @property
  def spin_marks(self): # -> list[str]:
    ...
  
  success = ...
  failed = ...
  interrupted = ...
  not_started = ...
  prefix_length = ...


class ConsoleAttr:
  """Console attribute and special drawing characters and functions accessor.

  Use GetConsoleAttr() to get a global ConsoleAttr object shared by all callers.
  Use ConsoleAttr() for abstracting multiple consoles.

  If _out is not associated with a console, or if the console properties cannot
  be determined, the default behavior is ASCII art with no attributes.

  Attributes:
    _ANSI_COLOR: The ANSI color control sequence dict.
    _ANSI_COLOR_RESET: The ANSI color reset control sequence string.
    _csi: The ANSI Control Sequence indicator string, '' if not supported.
    _encoding: The character encoding.
        ascii: ASCII art. This is the default.
        utf8: UTF-8 unicode.
        win: Windows code page 437.
    _font_bold: The ANSI bold font embellishment code string.
    _font_italic: The ANSI italic font embellishment code string.
    _get_raw_key: A function that reads one keypress from stdin with no echo.
    _out: The console output file stream.
    _term: TERM environment variable value.
    _term_size: The terminal (x, y) dimensions in characters.
  """
  _CONSOLE_ATTR_STATE = ...
  _ANSI_COLOR = ...
  _ANSI_COLOR_RESET = ...
  _BULLETS_UNICODE = ...
  _BULLETS_WINDOWS = ...
  _BULLETS_ASCII = ...
  def __init__(self, encoding=..., suppress_output=...) -> None:
    """Constructor.

    Args:
      encoding: Encoding override.
        ascii -- ASCII art. This is the default.
        utf8 -- UTF-8 unicode.
        win -- Windows code page 437.
      suppress_output: True to create a ConsoleAttr that doesn't want to output
        anything.
    """
    ...
  
  def Colorize(self, string, color, justify=...): # -> str:
    """Generates a colorized string, optionally justified.

    Args:
      string: The string to write.
      color: The color name -- must be in _ANSI_COLOR.
      justify: The justification function, no justification if None. For
        example, justify=lambda s: s.center(10)

    Returns:
      str, The colorized string that can be printed to the console.
    """
    ...
  
  def ConvertOutputToUnicode(self, buf): # -> str:
    """Converts a console output string buf to unicode.

    Mainly used for testing. Allows test comparisons in unicode while ensuring
    that unicode => encoding => unicode works.

    Args:
      buf: The console output string to convert.

    Returns:
      The console output string buf converted to unicode.
    """
    ...
  
  def GetBoxLineCharacters(self): # -> BoxLineCharactersUnicode | BoxLineCharactersAscii | BoxLineCharactersScreenReader:
    """Returns the box/line drawing characters object.

    The element names are from ISO 8879:1986//ENTITIES Box and Line Drawing//EN:
      http://www.w3.org/2003/entities/iso8879doc/isobox.html

    Returns:
      A BoxLineCharacters object for the console output device.
    """
    ...
  
  def GetBullets(self): # -> tuple[Literal['▪'], Literal['◆'], Literal['▸'], Literal['▫'], Literal['◇'], Literal['▹']] | tuple[Literal['■'], Literal['≡'], Literal['∞'], Literal['Φ'], Literal['·']] | tuple[Literal['o'], Literal['*'], Literal['+'], Literal['-']]:
    """Returns the bullet characters list.

    Use the list elements in order for best appearance in nested bullet lists,
    wrapping back to the first element for deep nesting. The list size depends
    on the console implementation.

    Returns:
      A tuple of bullet characters.
    """
    ...
  
  def GetProgressTrackerSymbols(self): # -> ProgressTrackerSymbolsUnicode | ProgressTrackerSymbolsAscii:
    """Returns the progress tracker characters object.

    Returns:
      A ProgressTrackerSymbols object for the console output device.
    """
    ...
  
  def GetControlSequenceIndicator(self): # -> str | None:
    """Returns the control sequence indicator string.

    Returns:
      The control sequence indicator string or None if control sequences are not
      supported.
    """
    ...
  
  def GetControlSequenceLen(self, buf): # -> int:
    """Returns the control sequence length at the beginning of buf.

    Used in display width computations. Control sequences have display width 0.

    Args:
      buf: The string to check for a control sequence.

    Returns:
      The control sequence length at the beginning of buf or 0 if buf does not
      start with a control sequence.
    """
    ...
  
  def GetEncoding(self): # -> str:
    """Returns the current encoding."""
    ...
  
  def GetFontCode(self, bold=..., italic=...): # -> str:
    """Returns a font code string for 0 or more embellishments.

    GetFontCode() with no args returns the default font code string.

    Args:
      bold: True for bold embellishment.
      italic: True for italic embellishment.

    Returns:
      The font code string for the requested embellishments. Write this string
        to the console output to control the font settings.
    """
    ...
  
  def GetRawKey(self): # -> str | None:
    """Reads one key press from stdin with no echo.

    Returns:
      The key name, None for EOF, <KEY-*> for function keys, otherwise a
      character.
    """
    ...
  
  def GetTermIdentifier(self): # -> str:
    """Returns the TERM environment variable for the console.

    Returns:
      str: A str that describes the console's text capabilities
    """
    ...
  
  def GetTermSize(self): # -> tuple[Literal[0], Literal[0]] | tuple[Any, Any] | tuple[Literal[80], Literal[24]]:
    """Returns the terminal (x, y) dimensions in characters.

    Returns:
      (x, y): A tuple of the terminal x and y dimensions.
    """
    ...
  
  def DisplayWidth(self, buf): # -> int:
    """Returns the display width of buf, handling unicode and ANSI controls.

    Args:
      buf: The string to count from.

    Returns:
      The display width of buf, handling unicode and ANSI controls.
    """
    ...
  
  def SplitIntoNormalAndControl(self, buf): # -> list[tuple[Any, str]] | list[Any]:
    """Returns a list of (normal_string, control_sequence) tuples from buf.

    Args:
      buf: The input string containing one or more control sequences
        interspersed with normal strings.

    Returns:
      A list of (normal_string, control_sequence) tuples.
    """
    ...
  
  def SplitLine(self, line, width): # -> list[Any]:
    """Splits line into width length chunks.

    Args:
      line: The line to split.
      width: The width of each chunk except the last which could be smaller than
        width.

    Returns:
      A list of chunks, all but the last with display width == width.
    """
    ...
  
  def SupportsAnsi(self): # -> bool:
    ...
  


class Colorizer:
  """Resource string colorizer.

  Attributes:
    _con: ConsoleAttr object.
    _color: Color name.
    _string: The string to colorize.
    _justify: The justification function, no justification if None. For example,
      justify=lambda s: s.center(10)
  """
  def __init__(self, string, color, justify=...) -> None:
    """Constructor.

    Args:
      string: The string to colorize.
      color: Color name used to index ConsoleAttr._ANSI_COLOR.
      justify: The justification function, no justification if None. For
        example, justify=lambda s: s.center(10)
    """
    ...
  
  def __eq__(self, other) -> bool:
    ...
  
  def __ne__(self, other) -> bool:
    ...
  
  def __gt__(self, other) -> bool:
    ...
  
  def __lt__(self, other) -> bool:
    ...
  
  def __ge__(self, other) -> bool:
    ...
  
  def __le__(self, other) -> bool:
    ...
  
  def __len__(self): # -> int:
    ...
  
  def __str__(self) -> str:
    ...
  
  def Render(self, stream, justify=...): # -> None:
    """Renders the string as self._color on the console.

    Args:
      stream: The stream to render the string to. The stream given here *must*
        have the same encoding as sys.stdout for this to work properly.
      justify: The justification function, self._justify if None.
    """
    ...
  


def GetConsoleAttr(encoding=..., reset=...): # -> ConsoleAttr | None:
  """Gets the console attribute state.

  If this is the first call or reset is True or encoding is not None and does
  not match the current encoding or out is not None and does not match the
  current out then the state is (re)initialized. Otherwise the current state
  is returned.

  This call associates the out file stream with the console. All console related
  output should go to the same stream.

  Args:
    encoding: Encoding override.
      ascii -- ASCII. This is the default.
      utf8 -- UTF-8 unicode.
      win -- Windows code page 437.
    reset: Force re-initialization if True.

  Returns:
    The global ConsoleAttr state object.
  """
  ...

def ResetConsoleAttr(encoding=...): # -> ConsoleAttr | None:
  """Resets the console attribute state to the console default.

  Args:
    encoding: Reset to this encoding instead of the default.
      ascii -- ASCII. This is the default.
      utf8 -- UTF-8 unicode.
      win -- Windows code page 437.

  Returns:
    The global ConsoleAttr state object.
  """
  ...

def GetCharacterDisplayWidth(char): # -> Literal[1, 0, 2]:
  """Returns the monospaced terminal display width of char.

  Assumptions:
    - monospaced display
    - ambiguous or unknown chars default to width 1
    - ASCII control char width is 1 => don't use this for control chars

  Args:
    char: The character to determine the display width of.

  Returns:
    The monospaced terminal display width of char: either 0, 1, or 2.
  """
  ...

def SafeText(data, encoding=..., escape=...): # -> str | None:
  rb"""Converts the data to a text string compatible with the given encoding.

  This works the same way as Decode() below except it guarantees that any
  characters in the resulting text string can be re-encoded using the given
  encoding (or GetConsoleAttr().GetEncoding() if None is given). This means
  that the string will be safe to print to sys.stdout (for example) without
  getting codec exceptions if the user's terminal doesn't support the encoding
  used by the source of the text.

  Args:
    data: Any bytes, string, or object that has str() or unicode() methods.
    encoding: The encoding name to ensure compatibility with. Defaults to
      GetConsoleAttr().GetEncoding().
    escape: Replace unencodable characters with a \uXXXX or \xXX equivalent if
      True. Otherwise replace unencodable characters with an appropriate unknown
      character, '?' for ASCII, and the unicode unknown replacement character
      \uFFFE for unicode.

  Returns:
    A text string representation of the data, but modified to remove any
    characters that would result in an encoding exception with the target
    encoding. In the worst case, with escape=False, it will contain only ?
    characters.
  """
  ...

def EncodeToBytes(data): # -> bytes:
  r"""Encode data to bytes.

  The primary use case is for base64/mime style 7-bit ascii encoding where the
  encoder input must be bytes. "safe" means that the conversion always returns
  bytes and will not raise codec exceptions.

  If data is text then an 8-bit ascii encoding is attempted, then the console
  encoding, and finally utf-8.

  Args:
    data: Any bytes, string, or object that has str() or unicode() methods.

  Returns:
    A bytes string representation of the data.
  """
  ...

def Decode(data, encoding=...): # -> str | None:
  """Converts the given string, bytes, or object to a text string.

  Args:
    data: Any bytes, string, or object that has str() or unicode() methods.
    encoding: A suggesting encoding used to decode. If this encoding doesn't
      work, other defaults are tried. Defaults to
      GetConsoleAttr().GetEncoding().

  Returns:
    A text string representation of the data.
  """
  ...

