"""
This type stub file was generated by pyright.
"""

"""This module has classes for tracing the execution of a Fire execution.

A FireTrace consists of a sequence of FireTraceElement objects. Each element
represents an action taken by Fire during a single Fire execution. An action may
be instantiating a class, calling a routine, or accessing a property.

Each action consumes args and results in a new component. The final component
is serialized to stdout by Fire as well as returned by the Fire method. If
a Fire usage error occurs, such as insufficient arguments being provided to call
a function, then that error will be captured in the trace and the final
component will be None.
"""
INITIAL_COMPONENT = ...
INSTANTIATED_CLASS = ...
CALLED_ROUTINE = ...
CALLED_CALLABLE = ...
ACCESSED_PROPERTY = ...
COMPLETION_SCRIPT = ...
INTERACTIVE_MODE = ...
class FireTrace:
  """A FireTrace represents the steps taken during a single Fire execution.

  A FireTrace consists of a sequence of FireTraceElement objects. Each element
  represents an action taken by Fire during a single Fire execution. An action
  may be instantiating a class, calling a routine, or accessing a property.
  """
  def __init__(self, initial_component, name=..., separator=..., verbose=..., show_help=..., show_trace=...) -> None:
    ...
  
  def GetResult(self): # -> None:
    """Returns the component from the last element of the trace."""
    ...
  
  def GetLastHealthyElement(self): # -> FireTraceElement:
    """Returns the last element of the trace that is not an error.

    This element will contain the final component indicated by the trace.

    Returns:
      The last element of the trace that is not an error.
    """
    ...
  
  def HasError(self): # -> bool:
    """Returns whether the Fire execution encountered a Fire usage error."""
    ...
  
  def AddAccessedProperty(self, component, target, args, filename, lineno): # -> None:
    ...
  
  def AddCalledComponent(self, component, target, args, filename, lineno, capacity, action=...): # -> None:
    """Adds an element to the trace indicating that a component was called.

    Also applies to instantiating a class.

    Args:
      component: The result of calling the callable.
      target: The name of the callable.
      args: The args consumed in order to call this callable.
      filename: The file in which the callable is defined, or None if N/A.
      lineno: The line number on which the callable is defined, or None if N/A.
      capacity: (bool) Whether the callable could have accepted additional args.
      action: The value to include as the action in the FireTraceElement.
    """
    ...
  
  def AddCompletionScript(self, script): # -> None:
    ...
  
  def AddInteractiveMode(self): # -> None:
    ...
  
  def AddError(self, error, args): # -> None:
    ...
  
  def AddSeparator(self): # -> None:
    """Marks that the most recent element of the trace used  a separator.

    A separator is an argument you can pass to a Fire CLI to separate args left
    of the separator from args right of the separator.

    Here's an example to demonstrate the separator. Let's say you have a
    function that takes a variable number of args, and you want to call that
    function, and then upper case the result. Here's how to do it:

    # in Python
    def display(arg1, arg2='!'):
      return arg1 + arg2

    # from Bash (the default separator is the hyphen -)
    display hello   # hello!
    display hello upper # helloupper
    display hello - upper # HELLO!

    Note how the separator caused the display function to be called with the
    default value for arg2.
    """
    ...
  
  def GetCommand(self, include_separators=...): # -> str:
    """Returns the command representing the trace up to this point.

    Args:
      include_separators: Whether or not to include separators in the command.

    Returns:
      A string representing a Fire CLI command that would produce this trace.
    """
    ...
  
  def NeedsSeparator(self): # -> bool | None:
    """Returns whether a separator should be added to the command.

    If the command is a function call, then adding an additional argument to the
    command sometimes would add an extra arg to the function call, and sometimes
    would add an arg acting on the result of the function call.

    This function tells us whether we should add a separator to the command
    before adding additional arguments in order to make sure the arg is applied
    to the result of the function call, and not the function call itself.

    Returns:
      Whether a separator should be added to the command if order to keep the
      component referred to by the command the same when adding additional args.
    """
    ...
  
  def __str__(self) -> str:
    ...
  
  def NeedsSeparatingHyphenHyphen(self, flag=...): # -> bool:
    """Returns whether a the trace need '--' before '--help'.

    '--' is needed when the component takes keyword arguments, when the value of
    flag matches one of the argument of the component, or the component takes in
    keyword-only arguments(e.g. argument with default value).

    Args:
      flag: the flag available for the trace

    Returns:
      True for needed '--', False otherwise.

    """
    ...
  


class FireTraceElement:
  """A FireTraceElement represents a single step taken by a Fire execution.

  Examples of a FireTraceElement are the instantiation of a class or the
  accessing of an object member.
  """
  def __init__(self, component=..., action=..., target=..., args=..., filename=..., lineno=..., error=..., capacity=...) -> None:
    """Instantiates a FireTraceElement.

    Args:
      component: The result of this element of the trace.
      action: The type of action (e.g. instantiating a class) taking place.
      target: (string) The name of the component being acted upon.
      args: The args consumed by the represented action.
      filename: The file in which the action is defined, or None if N/A.
      lineno: The line number on which the action is defined, or None if N/A.
      error: The error represented by the action, or None if N/A.
      capacity: (bool) Whether the action could have accepted additional args.
    """
    ...
  
  def HasError(self): # -> bool:
    ...
  
  def HasCapacity(self): # -> None:
    ...
  
  def HasSeparator(self): # -> bool:
    ...
  
  def AddSeparator(self): # -> None:
    ...
  
  def ErrorAsStr(self): # -> str:
    ...
  
  def __str__(self) -> str:
    ...
  


